package require BLT

# Some defaults and constants
set Client(count) 0
#set Client(colors) {red green blue purple orange}
set Client(stipple) {cross2 bdiagonal2 checker2 checker3}
set Client(tics) {{rcv bytes} {xmt bytes} {rcv errs} {xmt errs} {colls}}
set Client(pos) {1          9            3          11         14}

set Client(ip) 192.168.9.1
set Client(port) 55556




blt::bitmap define cross2 { 
#define cross2_width 8
#define cross2_height 8
static unsigned char cross2_bits[] = {
   0xff, 0x88, 0x88, 0x88, 0xff, 0x88, 0x88, 0x88};
}

blt::bitmap define bdiagonal2 { 
#define bdiagonal2_width 8
#define bdiagonal2_height 8
static unsigned char bdiagonal2_bits[] = {
   0x08, 0x04, 0x02, 0x01, 0x80, 0x40, 0x20, 0x10};
}

blt::bitmap define checker2 { 
#define checker2_width 8
#define checker2_height 8
static unsigned char checker2_bits[] = {
   0x33, 0x33, 0xcc, 0xcc, 0x33, 0x33, 0xcc, 0xcc};
}

blt::bitmap define checker3 { 
#define checker3_width 8
#define checker3_height 8
static unsigned char checker3_bits[] = {
   0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0};
}

################################################################
# proc getNetInfo {channel}--
#    Retrieves a set of network information from the socket
#    Parses the info, and creates a set of 'diff' index arrays
#    that are the difference between this value and the previous
#    value for a field.
#
# Arguments
#   channel	The channel to read data from
# 
# Results
#   Modifies the Client array.
#   Invokes processData to update the bar

proc getNetInfo {channel} {
    global Client

    gets $channel line

    # The first element is the name, but the second should
    #   be a number.  If it isn't (this is a line of column headers.)
    #   We'll skip out and wait for the next line of data.

    if {[catch {expr [lindex $line 1] * 2}]} {
	return
    }

    # Long integers may run into the ":" in the line label
    #  This gives us a space to parse on.
    regsub ":" $line " " line

    # The first entry is the device name.
    set name [lindex $line 0]

    global DataVector_x_${name}
    global DataVector_y_${name}
    if {![info exists DataVector_x_${name}]} {

	::blt::vector DataVector_x_${name}(5)
	::blt::vector DataVector_y_${name}(5)

	makeNewBarSet $name
    }

    # The DataVector_y vector holds the heights of the bars
    #   for a given data set.

    set vectorPos 0
    foreach pos $Client(pos) label $Client(tics) {
	set val [lindex $line $pos]
	if {[info exists Client($name.$label)]} {
	    set DataVector_y_${name}($vectorPos) \
	        [expr $val - $Client($name.$label)]
	    incr vectorPos
	}

	set Client($name.$label) $val
    }
    set Client(update) "Last Update: [clock format [clock seconds]\
      -format %H:%M:%S]"
}

################################################################
# proc makeNewBarSet {name}--
#  Creates a new set of bars, and reconfigures the barchart to hold them.
#
# Arguments
#  name		The name of the data associated with this set
# 
# Results
#   Creates a new DataVector global.
#   Modifies the barchart and existing DataVector_x_* data.

proc makeNewBarSet {name} {
    global Client
    $Client(barChart) element create $name -label "$name" \
        -stipple [lindex $Client(stipple) $Client(count)] \
	-fg black -bg white \
	-xdata DataVector_x_${name} \
        -ydata DataVector_y_${name}

    incr Client(count)

    # Make the bars 1/(n+1) wide - 
    # this creates a one bar-width space 
    # between the sets of data

    $Client(barChart) configure -barwidth [expr 1.0 / ($Client(count) + 1)]

    # The DataVector_x_* vector holds the location
    #  for the bars.

    # Tic's are marked on integer boundaries, so start at
    # -.5 to get tic labels centered on the data sets

    set item 0
    foreach v [info globals DataVector_x_*] {
	global $v
	for {set i 0} {$i < [llength $Client(tics)]} {incr i} {
	    set ${v}($i) [expr $i + $item / ($Client(count) + 1.0) -.5]
	}
	incr item
    }
}

################################################################
# proc getTicLabel {chart tic}--
#    Returns a textual label for the barchart
# Arguments
#   chart	The chart associated with this request
#   tic		The position of the tic being requested.

proc getTicLabel {chart tic} {
    global Client

    return [lindex $Client(tics) $tic]
}

# Make a quit button
button .b -text "Quit" -command "exit"
grid .b -row 0 -column 0

# And the update time label
label .l -textvar Client(update)
grid .l -row 0 -column 1

# Build a barchart
set Client(barChart) [::blt::barchart .bcht -width 600 -title\
  "Network Activity"]
$Client(barChart) axis configure x -command getTicLabel
$Client(barChart) axis configure y -logscale 1

grid $Client(barChart) -row 2 -column 0 -columnspan 3

# Open a client socket on the local system 
# (for testing purposes.)
set input [socket $Client(ip) $Client(port)]

# When data is available to be read, call getNetInfo
fileevent $input readable "getNetInfo $input"

# And wait for the fireworks to start.
